/*  Fischer's timed mutual exclusion algorithm for 2 processes. 
 
    The model maintains the mutual exclusion to the critical section 
    because A < B.  
    The default process count in this template is 2.  
    The process count can be overridden with option -Mk where 
    k is the number of processes. 
    How this template can be used ?
    
    There are the following specification files.  
    
    1. hf1.2.risk: The risk condition is that process 1 and 2 
       are in the critical section at the same time. 
       For example, to analyze the risk of 5 processes: 
    
        red -Tr -M5 hf.d hf1.2.risk 
       
       The answer is no risk.  
*/ 
#define A 10
#define B 19 

process count = 2;

global pointer lock; 

local clock x; 

mode idle (true) {
  when (lock == NULL) may x= 0; goto ready; 
}

mode ready (true) {
  when (x < A) may x= 0; lock= P; goto waiting; 
}

mode waiting (true) { 
  when (x > B && lock == P) may goto critical; 
  when (lock != P) may goto idle; 
}

mode critical (true) { 
  when (true) may lock = NULL; goto idle; 
} 

initially lock == NULL && forall pi:1..#PS, (idle[pi] && x[pi] == 0); 



