/* 
Fischer's timed mutual exclusion algorithm for 2 processes. 
 
    The model can reach the risk condition  because A >= B.  
    A typical counter example looks like this. 
    
    (lock=0,idle,idle)
     |
     |(p1,t1)#sx1
     |
     V
    (lock=0,ready,idle,0<=x1)
     |
     |(p2,t1)#sx6
     |
     V
    (lock=0,ready,ready,0<=x2<=x1)
     |
     |(p1,t2,x1<100)#sx2
     |
     V
    (lock=1,wait,ready,0<=x1<=x2)
     |
     |(p1,t3,x1>19)#sx3
     |
     V
    (lock=1,critical,ready,19<x1<=x2)
     |
     |(p2,t2,x2<100)#sx7
     |
     V
    (lock=2,critical,wait,0<=x2&&19<x1&&x2+19<x1)
     |
     |(p2,t3)#sx8
     |
     V
    (lock=2,critical,critical, 19<x2&&38<x1&&x2+19<x1)
    
    The default process count in this template is 2.  
    The process count can be overridden with option -Mk where 
    k is the number of processes. 
    How this template can be used ?
    
    There are the following specification files.  
    
    1. hf1.2.risk: The risk condition is that process 1 and 2 
       are in the critical section at the same time. 
       For example, to analyze the risk of 5 processes: 
    
        red -Tr -M5 -C hfn.d hf1.2.risk 
       
       The answer is that risk condition is reachable.  
       Option -C geneates the counter example. 
*/ 
#define	A 100
#define B 19 

process count = 2;

global pointer lock; 

local clock x; 

mode idle (true) {
/*1*/  when (lock == NULL) may x= 0; goto ready; 
}

mode ready (true) {
/*2*/  when (x < A) may x= 0; lock= P; goto waiting; 
}

mode waiting (true) { 
/*3*/  when (x > B && lock == P) may goto critical; 
/*4*/  when (lock != P) may goto idle; 
}

mode critical (true) { 
/*5*/  when (true) may lock = NULL; goto idle; 
} 

initially 
lock == NULL && forall i:1..#PS, (idle[i] && x[i] == 0); 

risk critical[1] && critical[2];


